#ifndef _KSEMEM_H_
#define _KSEMEM_H_	1

#include <linux/types.h>
#include <linux/const.h>
#include "uapi/semem/semem_com.h"
#include "uapi/semem/semem_err.h"

#define SECURE_KEY_LEN	256


#define RULE_INVALID			(1ul << (__BITS_PER_LONG - 1))
#define RULE_REMOVE				(1ul << (__BITS_PER_LONG - 2))
#define RULE_RESET				(1ul << (__BITS_PER_LONG - 3))

#if 0
/*
*	Definition of memory attribute
*/
#define MEM_PROT_NONE		0			/* disable protect */
#define MEM_PROT_READ		1			/* not readable, hide */
#define MEM_PROT_WRITE		2			/* not writable, read-only */

#define RULE_MEM_PROT_MASK		0xFFUL

#define SEMEM_TEST_BIT(flag, bit)	(((flag) & (bit)) == (bit))
#define SEMEM_TEST_BIT_MISC(flag, misc_bit)	SEMEM_TEST_BIT(flag, misc_bit)
#endif

enum ksemem_notify_nr{
	KSEMEM_CHIP_KEY_ATTACK,
	KSEMEM_DRIVER_KEY_ATTACK,
	KSEMEM_SECURE_AREA_ATTACK
};

struct ksemem_event{
	int event;
	pid_t pid;			/* task that is doing the attack */
	unsigned long count;
};

struct security_mem_ops{
	char *ops_name;
	int (*rule_set)(unsigned char *key, struct semem_range *range);
	int (*rule_remove)(unsigned char *key, struct semem_range *range);
	int (*rule_set_prev)(struct semem_range *range, void *arg);
	int (*rule_set_post)(struct semem_range *range, void *arg);
	int (*rule_remove_prev)(struct semem_range *range, void *arg);
	int (*rule_remove_post)(struct semem_range *range, void *arg);
	int (*rule_disable)(unsigned char *key, struct semem_range *range);
	int (*rule_enable)(unsigned char *key, struct semem_range *range);
	int (*rule_attr_update)(unsigned char *key, struct semem_range *range);
	int (*rule_attach)(struct semem_range *range);
	int (*rule_detach)(struct semem_range *range);
	int (*rule_get_log)(unsigned char *key, struct semem_range *range, unsigned int *log_nr,
						unsigned int *log_entry_nr, unsigned int log_limit, struct log_struct __kernel *log_buf);
	int (*rule_get_log_count)(unsigned char *key, struct semem_range *range, unsigned int *log_nr);
};


int ksemem_secure_key_init(unsigned char *key);


/*
*	ksemem_rule_set will set a new rule with a start address @virt_addr,
*   a memory length @len and memory attribute @flag
*
*	in	  :		virt_addr					the start address of memory range
*				len							the length of memory range
*				flag						the attribute of memory range
*
*	out	  :     reg_nr						will return the index within the register table.
*
*   return:		0  succeed
*				-ERR_SEMEM_NOT_PERMIT		the caller is not trusted
*				-ERR_SEMEM_INVALID			parameter is invalid
*				-ERR_SEMEM_FAILED			rule cannot be set
*				-ERR_SEMEM_RULE_MAX			there is no avaliable rule register
*/
extern int ksemem_rule_set(unsigned char *key, struct semem_range *range);

extern int ksemem_rule_remove(unsigned char *key, struct semem_range *range);

extern int ksemem_rule_disable(unsigned char *key, struct semem_range *range);

extern int ksemem_rule_enable(unsigned char *key, struct semem_range *range);

extern int ksemem_rule_attribute_update(unsigned char *key, struct semem_range *range);

extern int ksemem_rule_attach(struct semem_range *range);

extern int ksemem_rule_detach(struct semem_range *range);

extern int ksemem_rule_replace(unsigned char *key, struct semem_range *range, struct semem_range *replaced_range);

extern int ksemem_rule_replaceable(unsigned char *key, struct semem_range *range, struct semem_range *replaced_range, unsigned int *replace_nr);

extern int ksemem_rule_get_log(unsigned char *key, struct semem_range *range, unsigned int *log_nr,
						unsigned int *log_entry_nr, unsigned int log_limit, struct log_struct __kernel *log_buf);

extern int ksemem_rule_get_log_count(unsigned char *key, struct semem_range *range, unsigned int *log_nr);

extern int ksemem_event_notify_register(unsigned char *key, struct notifier_block *nb);

extern int ksemem_addr_to_system(struct mm_struct *mm, unsigned long virt_addr, unsigned long *system_addr);

extern int ksemem_sys_to_dram(unsigned long system_addr, unsigned long *dram_addr);

extern int ksemem_addr_lookup(struct mm_struct *mm, unsigned long virt_addr, pte_t **pte_p_p, int *level);

extern int ksemem_pte_to_system(unsigned long virt_addr, pte_t *ptep, unsigned long *phys_addr, int level);

extern int ksemem_is_consecutive_range(struct mm_struct *mm, unsigned long virt_addr, unsigned long len,
											unsigned long *system_addr);

extern void security_mem_ops_register(struct security_mem_ops *mem_ops_handler);

extern void security_mem_ops_unregister(void);

extern void security_default_mem_ops_register(void);

extern int ksemem_rule_remove_ext(unsigned char *key, struct semem_range *range,
									int (*prev_handler)(struct semem_range *__range, void *__arg),
									int (*post_handler)(struct semem_range *__range, void *__arg),
									void *arg);

extern int ksemem_rule_set_ext(unsigned char *key, struct semem_range *range,
								int (*prev_handler)(struct semem_range *__range, void *__arg),
								int (*post_handler)(struct semem_range *__range, void *__arg),
								void *arg);

#endif


