#!/bin/bash

ARCH=`uname -m`
Arch=`echo $ARCH | sed -e s/i.86/x86/ -e s/x86_64/x86/ -e s/amd64/x86/ -e s/aarch64.*/arm64/ -e s/mips64el/mips/`
usage() {
	echo -e "\e[01;31m`basename $0`\e[00m [-h | --help] [-t | --test] [--baseonly] [--full] [--without-biv] [--without-debug] [[-c | --config] config]"
	echo -e "    \e[01;34m--h | --help\e[00m\tShow This Help"
	echo -e "    \e[01;34m--t | --test\e[00m\tBuild Kernel With '.fortest' Lable"
	echo -e "    \e[01;34m--S | --source\e[00m\tCreate Kernel source rpm package"
	echo -e "    \e[01;34m--baseonly\e[00m\t\tBuild Base Kernel Only, Without BIV and Debug Version"
	echo -e "    \e[01;34m--bivonly\e[00m\t\tBuild Benchmark Improve Version Only, Without Standard and Debug Version"
	echo -e "    \e[01;34m--debugonly\e[00m\t\tBuild Debug Kernel Only, Without Standard and BIV version"
	echo -e "    \e[01;34m--full\e[00m\t\tBuild Full Kernel"
	echo -e "    \e[01;34m--without-biv\e[00m\tBuild Full Kernel Without Benchmark Improve Version"
	echo -e "    \e[01;34m--without-debug\e[00m\tBuild Full Kernel Without Debug Version"
	echo -e "    \e[01;34m--without-up\e[00m\tBuild Full Kernel Without Standard Version"
	echo -e "    \e[01;34m-c | --config\e[00m\tBuild Test Kernel With Specify Config"
if [ ${Arch}x = "mips"x ]; then
	echo -e "    $_HBLUE--kdump$_END\t\tBuild Kernel Used By Kdump."
fi
}

if [ $# -lt 1 ]; then
	usage
	exit -1;
fi

BUILDUP=true
BUILDBIV=true
BUILDDEBUG=true
BUILDTEST=false
BUILDKDUMP=false
TESTLAB=".fortest"

ARGS=`getopt -o htc:S -a --long help,test,baseonly,bivonly,debugonly,full,without-biv,without-debug,without-up,config,kdump,source -- "$@"`
if [ $? != 0 ]; then
	echo "Terminating..."
	usage && exit 1
fi

eval set -- "${ARGS}"

while true
do
	case $1 in
		--baseonly)
			BUILDBIV=false
			BUILDDEBUG=false
			shift;;
		--bivonly)
			BUILDUP=false
			BUILDDEBUG=false
			shift;;
		--debugonly)
			BUILDBIV=false
			BUILDUP=false
			shift;;
		--full)
			shift;;
		--without-biv)
			BUILDBIV=false
			shift;;
		--without-debug)
			BUILDDEBUG=false
			shift;;
		--without-up)
			BUILDUP=false
			shift;;
		-t|--test)
			BUILDTEST=true
			shift;;
		-S|--source)
			CREATE_SOURCE_PKG="-bs"
			shift;;
		-c|--config)
			case "$2" in
				"")
					echo "Option c, no argument";
					exit -1;;
				*)
					BUILDBIV=false
					BUILDDEBUG=false
					Config=$2
					shift 2;;
			esac;;
		--)
			shift
			break;;
		--kdump)
			if [ ${Arch}x = "mips"x ]; then
				BUILDUP=true
				BUILDBIV=false
				BUILDDEBUG=false
				BUILDKDUMP=true
			else
				echo "Option --kdump only supports mips."
				exit -1
			fi
			shift;;
		-h|--help|*)
			usage
			exit -1;
	esac
done

for arg in $@
do
	echo "Ignore Unknow Args $arg.."
	usage && exit -1
done

TOPDIR=`pwd`/rpmbuild

if [ "$Config" != "" ]; then
	CLEAN="-e $Config"
	BUILDTEST=true
	\cp $Config arch/${Arch}/configs/kylinos_generic_defconfig
	if [ $? != 0 ]; then
		exit -1;
	fi
fi

git clean -xdf -e nohup.out $CLEAN
rm -rf rpmbuild/BUILD
rm -rf rpmbuild/BUILDROOT

rm -rf rpmbuild/linux-4.19.90
mkdir rpmbuild/linux-4.19.90
make mrproper -j 16

rsync -av --exclude rpmbuild --exclude .git . rpmbuild/linux-4.19.90

#prepare linux-4.19.90.tar.gz
cd rpmbuild

tar -zcvf linux-4.19.90.tar.xz linux-4.19.90
cd ../
rm -rf rpmbuild/SOURCES/linux-4.19.90.tar.xz
mv rpmbuild/linux-4.19.90.tar.xz rpmbuild/SOURCES

#do building
BUILD_ARGS=""

if [[ $BUILDBIV == false ]]; then
	BUILD_ARGS="$BUILD_ARGS --without biv"
fi
if [[ $BUILDDEBUG == false ]]; then
	BUILD_ARGS="$BUILD_ARGS --without debug"
fi
if [[ $BUILDUP == false ]]; then
	BUILD_ARGS="$BUILD_ARGS --without up"
fi

if [[ $BUILDKDUMP == true ]]; then
	BUILD_ARGS="$BUILD_ARGS --with kdump"
fi

if [[ $BUILDTEST == true ]]; then
	rpmbuild ${CREATE_SOURCE_PKG:--ba} rpmbuild/SPECS/kernel.spec --define="_topdir $TOPDIR" ${BUILD_ARGS} --define="custom ${TESTLAB}"
else
	rpmbuild ${CREATE_SOURCE_PKG:--ba} rpmbuild/SPECS/kernel.spec --define="_topdir $TOPDIR" ${BUILD_ARGS}
fi

#clean work
if [ $? -eq 0 ]; then
	echo "building rpm success..."
	rm -rf rpmbuild/kernel
else
	echo "building failed..."
	exit
fi

if [ ! -z $CREATE_SOURCE_PKG ]; then
	exit;
fi

#archive RPMS
RPMDIR=${TOPDIR}/RPMS/${ARCH}
KVRELEASE=4.19.90-kernel-${ARCH}
rm -rf rpmbuild/$KVRELEASE*
mkdir rpmbuild/$KVRELEASE

for pack in `ls $RPMDIR/${rpmpkg}*.rpm | grep -v debuginfo-common`
do
	cp $pack rpmbuild/$KVRELEASE
done

if [[ $BUILDKDUMP != false ]]; then
	mkdir -p rpmbuild/$KVRELEASE/kdump-version
	mv rpmbuild/$KVRELEASE/*kernel-kdump-*.rpm rpmbuild/$KVRELEASE/kdump-version
	VERSIONS="$VERSIONS kdump-version"
fi

#clean
cd rpmbuild
tar -zcvf ${KVRELEASE}.tar.gz $KVRELEASE
rm -rf $KVRELEASE
echo -e "Archive all builded rpm packages in: \nrpmbuild/${KVRELEASE}.tar.gz"
cd ..
